/*
 * Copyright (C) 2019 Computer Graphics Group, University of Siegen
 * Written by Martin Lambers <martin.lambers@uni-siegen.de>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef QV_OVERLAY_HPP
#define QV_OVERLAY_HPP

#include <memory>

#include "textureholder.hpp"


class QImage;
class QPainter;

class Overlay {
private:
    std::shared_ptr<TextureHolder> _textureHolder;

protected:
    QImage* _image;
    QPainter* _painter;

    void prepare(int widthInPixels, int heightInPixels);
    void fixFormat(int opaqueBlockX = -1, int opaqueBlockY = -1,
            int opaqueBlockW = -1, int opaqueBlockH = -1);
    void uploadImageToTexture();

public:
    Overlay();
    virtual ~Overlay();

    int heightInPixels() const { return _image->height(); }
    unsigned int texture() const { return _textureHolder->texture(0); }
};

#endif
